* First Order Logarithmic Domain Lowpass Filter
*
* Copyright: (c) 2014 Lars Hedrich, Felix Salfelder
* Author: Lars Hedrich
* License: CC-BY-SA
*>spice


.param v_suply=2 v_bias=1 i_bias=3U beta=50 c_int=20P

* Mos models
.model n_model NMOS level=1 vto=0.5  lambda=0.01 kp='2.5*20.718U'
.model p_model PMOS level=1 vto=-0.5 lambda=0.02 kp='1.0*20.718U'

* Bipolar model
.model qnd NPN bf=50 tf=0.1N tr=10N cje=0.9P cjc=1.5P vjc=0.85 vaf=50

.subckt log_lp uin uout nref
Vvdd  vdd  0    DC 'v_suply'

*Input voltage to current
Gin   0    iinp uin  0    1U
Rin   uin 0 1G

* logdomain current mode integrator
* with current (iout) feedback to build a low pass filter
Xcomn iinn vinn           compressor
Xcomp iinp vinp           compressor
Xint  vinn vinp vout      integrator
Xexp  vout iout           expandor

* Mirroring the currents back and
MN1   iout  iout  0  0  n_model w=20U l=2U
MN2   iim   iout  0  0  n_model w=20U l=2U

MP1   iim   iim  vdd  vdd  p_model w=20U l=2U
MP2   iinn  iim  vdd  vdd  p_model w=20U l=2U
MP3   ioutp iim  vdd  vdd  p_model w=20U l=2U

*measuring output current and convert back to output as voltage
Riu   ioutp 0   1
Eout  uout 0 ioutp 0 1Meg
Rout  uout 0 100G

.ends log_lp


***** log_compressor *****

.subckt compressor iin vout
Vvb   base 0    DC 'v_bias'
Mm    vout iin  0    0    n_model w=20U l=2U
Qq    iin  base vout 0    qnd area=0.01
.ends


***** exp_expandor *****

.subckt expandor vin iout
Vvb   base 0    DC 'v_bias'
Vvdd  vdd  0    DC 'v_suply'
Qq1   iin  base vin  0    qnd area=0.01
Mm6   iin  iin  vdd  vdd  p_model w=20U l=2U
Mm7   iout iin  vdd  vdd  p_model w=20U l=2U
.ends


***** log domain integrator *****

.subckt integrator vinn vinp vout
Iib   vdd  n2   DC 'i_bias'
Ibeta vdd  base DC 'i_bias/beta'
Vvdd  vdd  0    DC 'v_suply'
Qq3   n1   base vinn 0    qnd area=0.01
Qq4   base base vinp 0    qnd area=0.01
Qq5   n2   base vout 0    qnd area=0.01
Mm3   n1   n1   vdd  vdd  p_model w=20U l=2U
Mm4   base n1   vdd  vdd  p_model w=20U l=2U
Mm5   vout n2   0    0    n_model w=20U l=2U
Cc1   base 0    c_int
.ends

* centering levels a bit
.subckt lp uin uout nref
V1 uin_ uin 4
V2 uout_ uout 4.1
Xlp uin_ uout_ nref log_lp
.ends
